; ************************
;        VGAGEN2
; ************************

.INCLUDE "m48def.inc" 


.equ F_CPU			= 20000000			; CPU Frequenz in Hz
.equ F_KHz_CPU		= F_CPU/1000		; CPU Frequenz in KHz
.equ CTC_WERT		= 31750*F_KHz_CPU/1000000


; Verwaltung der Ports
; ---------------------

.equ 	port_control_b	= portb
.equ 	port_control_c	= portc
.equ 	port_adr_hig	= portd

.equ 	ddr_control_b	= ddrb
.equ 	ddr_control_c	= ddrc
.equ 	ddr_adr_hig		= ddrd




; *** port_control_b ***
.equ	LED_GE		= 0			; LED_GE -> gelbe LED
.equ 	CE			= 1			; /CE fuer SRAM



; *** port_control_c ***
.equ 	HSYNC		= 0			; Horizontale Synchronimpulse Pin 13
.equ 	VSYNC		= 1			; Vertikale Synchronimpulse Pin 14

.equ	START_CNT	= 2			; /START_CNT = Start des Zaehlers -> Low active
.equ	RESET_CNT	= 3			; RESET_CNT = Ruecksetzen des Zaehlers auf 0 -> High active

.equ 	WE			= 4			; /WE fuer SRAM
.equ 	OE			= 5			; /OE fuer SRAM




; VERTICAL LINE CONTROL 
.equ 	VL_RESET	= 0			; 
.equ 	VL_VER_IMP	= 1			;
.equ 	VL_OUT_VA	= 2			;
.equ 	VL_INC_VA	= 3			;
.equ 	VL_MEMREADY	= 4			;




; *** Register R16 bis R25 ***

; 1. Register in der Hauptroutine

.def temp1    = r16		; TEMP REGISTER 1 nur fuer Hauptroutine


; 2. Register in der Interruptroutine

.def temp1i   = r19		; TEMP REGISTER 1 nur fuer Interruptroutine
.def VER_ADR  = r20





;****************************************************************************
;	Interrupt Vektoren
;****************************************************************************


.cseg


.org 0
	rjmp	RESET
.org oc1aaddr
	rjmp	VIDEO		;2



RESET:


; STACK POINTER
	ldi		temp1,low(ramend)
	out		spl,temp1
	ldi		temp1,high(ramend)
	out		sph,temp1





; ****************************************************************************************
; **** Setup IO-Port B
; ****************************************************************************************

	sbi		ddr_control_b, CE		; SRAM CHIP ENABLE
	sbi		ddr_control_b, LED_GE




; ****************************************************************************************
; **** Setup IO-Port C
; ****************************************************************************************

	sbi		ddr_control_c, HSYNC 		; HORIZONTAL SYNC PULSE
	sbi		ddr_control_c, VSYNC 		; VERTICAL SYNC PULSE
	sbi		ddr_control_c, WE			; SRAM WRITE ENABLE
	sbi		ddr_control_c, OE			; SRAM OUTPUT ENABLE

	sbi		ddr_control_c, START_CNT	; START_CNT
	sbi		ddr_control_c, RESET_CNT	; RESET_CNT



; ****************************************************************************************
; **** Setup IO-Port D
; ****************************************************************************************

	sbi		ddr_adr_hig, 0		; ADDRESS BIT 09
	sbi		ddr_adr_hig, 1		; ADDRESS BIT 10
	sbi		ddr_adr_hig, 2		; ADDRESS BIT 11
	sbi		ddr_adr_hig, 3		; ADDRESS BIT 12
	sbi		ddr_adr_hig, 4		; ADDRESS BIT 13
	sbi		ddr_adr_hig, 5		; ADDRESS BIT 14
	sbi		ddr_adr_hig, 6		; ADDRESS BIT 15
	sbi		ddr_adr_hig, 7		; ADDRESS BIT 16






; Lschen der lokalen Datenregister

	clr		xl
	clr		xh
	clr		yl
	clr		yh
	clr		zl
	clr		zh
 
	clr		temp1
	clr		temp1i
	clr		VER_ADR




; definiertes Setzen der Control-Pins

	cbi		port_control_c, HSYNC
	cbi		port_control_c, VSYNC

	cbi		port_control_c, START_CNT
	cbi		port_control_c, RESET_CNT

	sbi		port_control_c, WE
	sbi		port_control_c, OE

	sbi		port_control_b, CE
	cbi		port_control_b, LED_GE



; ****************************************************************************************
; **** STARTUP SEQUENCE
; ****************************************************************************************


; CTC Modus (Clear Timer on Compare Match mode) 
; ---------------------------------------------

; genutzt wird der 16-Bit Timer1

; TCCR1B = Timer/Counter control Register B des Timers1
; | ICNC1 | ICES1 | leer | WGM13 | WGM12 | CS12 | CS11 | CS10 |
; CS12  CS11  CS10  
;   0     0     1  = CPU-Takt ohne Teiler
;
; WGM12 = // CTC-Modus, TOP = OCR1A
;
; Timer 1, ein 16-Bit-Timer, wird mit einem Vorteiler von 1 betrieben. 
; Dadurch wird erreicht, dass der Timer mit hchster Zeitauflsung arbeiten kann. 
; Bei jedem Ticken des Systemtaktes von 4 MHz wird auch der Timer um 1 erhht. 
; Zustzlich wird noch das WGM12-Bit bei der Konfiguration gesetzt. 
; Dadurch wird der Timer in den CTC-Modus gesetzt. 
; Dabei wird der Inhalt des Timers hardwaremig mit dem Inhalt des OCR1A-Registers verglichen. 
; Stimmen beide berein, so wird der Timer auf 0 zurckgesetzt und im nchsten Taktzyklus ein OCIE1A-Interrupt ausgelst.
; Dadurch ist es mglich, exakt die Anzahl an Taktzyklen festzulegen, die von einem Interrupt zum nchsten vergehen sollen.


	ldi		temp1, (1<<CS10 | 1<<WGM12)
	sts		tccr1b, temp1




; Einstellen der Interrupt-Zeit
; alle 31,75 us oder 31750 ns 

	ldi		temp1, high(CTC_WERT)
	sts		ocr1ah, temp1
	ldi		temp1, low(CTC_WERT)
	sts		ocr1al, temp1



; TIMSK 
; | TICIE1 | OCIE1A | OCIE1B | TOIE1 |  
;
; TICIE1 - Timer 1 Input Capture Interrupt Enable
; OCIE1A - Timer 1 Output Compare A Match Interrupt Enable
; OCIE1B - Timer 1 Output Compare B Match Interrupt Enable
; TOIE1  - Timer 1 Overflow Interrupt Enable
;
; OCIE1A - Timer/Counter 1 Vergleichszahl A Interrupt - 1: Int bei Erreichen Stand in A
;
; SET TIMER1 MATCH INTERRUPT
	ldi		temp1, (1<<OCIE1A)
	sts		timsk1, temp1




; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

; Einschalten des Interrupts 
; damit beginnt sofort (nach dem naechsten Befehl) die ISR mit der Anzeige des Bildes
	sei



; ****************************************************************************************
; **** MAIN PROGRAM LOOP
; ****************************************************************************************


MainLoop:

	cbi		port_control_b, LED_GE

	rjmp	MainLoop	






;****************************************************************************
;****************************************************************************
;	VIDEO - Interruptroutine
;****************************************************************************
;****************************************************************************



 

; ****************************************************************************************
; **** VIDEO RENDERING INTERRUPT / R2=SOUND FREQUENCY / RETURNS R25=MEMREADY
; ****************************************************************************************
VIDEO:





; ****************************************************************************************
; **** HORIZONTAL AND VERTICAL SYNC = 76 CYCLES
; ****************************************************************************************



; (0)


; SAVE STATUS REGISTER
	in		temp1i,sreg		;1
	push	temp1i			;2







; EQUALIZE INTERRUPT LATENCY
; Angleichen der Interrupt Verborgenheit

	lds		temp1i,tcnt1l	;2
	cpi		temp1i,10		;1
	brlo	LATFIX1			;1/2

LATFIX1:
	cpi		temp1i,11		;1
	brlo	LATFIX2			;1/2

LATFIX2:
	cpi		temp1i,12		;1
	brlo	LATFIX3			;1/2

LATFIX3:
	cpi		temp1i,13		;1
	brlo	LATFIX4			;1/2

LATFIX4:









	sbi		port_control_b, LED_GE


; HORIZONTAL SYNC LOW
	cbi		port_control_c, HSYNC	;2



; bei 20 MHz
; HSP =  3800 nS / 50 nS =  76
; HBP =  1540 nS / 50 nS =  31
; HPX = 25600 nS / 50 nS = 512
; HFP =   640 nS / 50 nS =  12


; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
; |
; ---------------------------------------------
; ab hier 76 Zyklen HSP

; (0)


; SET POINTER TO VLINE TABLE
	ldi		zl,low(2*VLINE)		;1
	ldi		zh,high(2*VLINE)	;1



; (2)


; SET VLINE TABLE POSITION
	add		zl,yl				;1
	adc		zh,yh				;1


; READ VLINE TABLE BYTE
	lpm		temp1i,z			;3


; (7)


; INCREMENT VERTICAL ADDRESS
	sbrs	temp1i,VL_INC_VA		;1/2
	inc		VER_ADR					;1


; RESET VERTICAL ADDRESS
	sbrc	temp1i,VL_RESET			;1/2	; ueberspringen von ser, wenn
	ser		VER_ADR					;1		; Bit0=0 in temp1i


; INCREMENT VLINE COUNTER
	adiw	yh:yl,1					;1


; (13)


; RESET VLINE COUNTER
	sbrc	temp1i,VL_RESET			;1/2
	clr		yl						;1
	sbrc	temp1i,VL_RESET			;1/2
	clr		yh						;1


; (17)


; VERTICAL SYNC CONTROL
	sbrc	temp1i, VL_VER_IMP		;1/2
	cbi		port_control_c, VSYNC	;2
	sbrs	temp1i, VL_VER_IMP		;1/2
	sbi		port_control_c, VSYNC	;2

; (22)


; HORIZONTAL SYNC HIGH
	sbi		port_control_c, HSYNC	;2

 

; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;       |
; ---------------------------------------------
; ab hier 31 Zyklen HBP


; (0)



; ****************************************************************************************
; **** HORIZONTAL BACK PORCH = 36 CYCLES
; ****************************************************************************************

; SET VERTICAL ADDRESS
	sbrc	temp1i,VL_OUT_VA		;1/2
	out		port_adr_hig,VER_ADR	;1

; CONTROL ACTIVE LINES
	sbrs	temp1i,VL_OUT_VA		;1/2
	rjmp	NOVID					;2

; (4)

; Ruecksetzen des Zaehlers
	sbi		port_control_c, RESET_CNT	; 2
	nop									; 1
	nop									; 1
	cbi		port_control_c, RESET_CNT	; 2

; (10)

; SET SRAM TO READ

	cbi		port_control_b, CE			; 2
	nop									; 1
	nop									; 1
	cbi		port_control_c, OE			; 2

; (16)

; (20)

; Starten des Zaehlers
	sbi		port_control_c, START_CNT	; 2

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	

; (100)
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
; (200)







	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1


;(300)




	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
;(400)


	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop												; 1

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	


	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	

;(480)

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	

;(500)

	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	nop									; 1
	nop	
	


	cbi		port_control_c, START_CNT	; 2
 
	sbi		port_control_c, WE
	sbi		port_control_c, OE

	sbi		port_control_b, CE
	cbi		port_control_b, LED_GE



; ****************************************************************************************
; **** HORIZONTAL ACTIVE LINE = 512 CYCLES
; ****************************************************************************************





; ---------------------------------------------
;  HSP - HBP - HPX   -    HFP
;   76   31   512         12
;__     ____xxxxxxxxxxxx_____
; |     |
; -------
; 1     2   3           4   5
;           |
; ---------------------------------------------
; ab hier 512 Zyklen HPX






; ****************************************************************************************
; **** HORIZONTAL FRONT PORCH = 12 CYCLES
; ****************************************************************************************



; BLANKING LINES EXIT POINT
NOVID:


; RESTORE STATUS REGISTER
	pop		temp1i			 ;2
	out		sreg,temp1i		 ;1


; RETURN FROM INTERRUPT
	reti					 ;4


; (35)






; ****************************************************************************************
; **** VERTICAL LINE LOOKUP TABLE
; ****************************************************************************************

.equ 	BY_OT		= (1<<VL_OUT_VA)
.equ 	BY_IN_OT	= (1<<VL_INC_VA | 1<<VL_OUT_VA)
.equ 	BY_MR		= (1<<VL_MEMREADY)
.equ 	BY_MR_VI	= (1<<VL_MEMREADY | 1<<VL_VER_IMP)
.equ 	BY_ZERO		= 0
.equ 	BY_RT		= (1<<VL_RESET)

VLINE:

.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT 
.db BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT,BY_OT,BY_IN_OT

.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR_VI,BY_MR_VI,BY_MR
.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR
.db BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR,BY_MR

.db BY_MR,BY_ZERO,BY_RT,BY_ZERO







































